--[[=============================================================================
    Get, Handle and Dispatch message functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end

--[[=============================================================================
    GetMessage()
  
    Description:
    Used to retrieve a message from the communication buffer. Each driver is
    responsible for parsing that communication from the buffer.
  
    Parameters:
    None
  
    Returns:
    A single message from the communication buffer
===============================================================================]]
function GetMessage()
	local message, pos
	
	--TODO: Implement a string using Lua captures and patterns which 
	--		will be used by string.match to parse out a single message
	--      from the receive buffer(gReceiveBuffer).
	--		The example shown here will return all characters from the beginning of 
	--		the gReceiveBuffer up until but not including the first carriage return.
	local pattern = "-(.-)\r()"
	
	if (gReceiveBuffer:len() > 0) then
		message, pos = string.match(gReceiveBuffer, pattern)
		LogTrace("device message: "..message)
		if (message == nil) then
			--LOG:Info("Do not have a complete message")
			return ""
		end
		gReceiveBuffer = gReceiveBuffer:sub(pos)		
	end

	return message
	
end

--[[=============================================================================
    HandleMessage(message)]

    Description
    This is where we parse the messages returned from the GetMessage()
    function into a command and data. The call to 'DispatchMessage' will use the
    'name' variable as a key to determine which handler routine, function, should
    be called in the DEV_MSG table. The 'value' variable will then be passed as
    a string parameter to that routine.

    Parameters
    message(string) - Message string containing the function and value to be sent to
                      DispatchMessage

    Returns
    Nothing
===============================================================================]]
function HandleMessage(message)
	--TODO: Implement a string using Lua captures and patterns which 
	--		will be used by string.match to parse the message
	--      into a name / value pair.
	--		The example shown here will return all alpha characters 
	--		up to the first non-alpha character and store them in the "name" variable
	--		the remaining characters will be returned and stored in the "value" variable.
	local pattern =  "^(.+)%.(.+)"

	local name, value, pos = string.match(message, pattern)
	name = name or message
	value = value or ""	

	DispatchMessage(name, value)
	
end

--[[=============================================================================
    DispatchMessage(MsgKey, MsgData)

    Description
    Parse routine that will call the routines to handle the information returned
    by the connected system.

    Parameters
    MsgKey(string)  - The function to be called from within DispatchMessage
    MsgData(string) - The parameters to be passed to the function found in MsgKey

    Returns
    Nothing
===============================================================================]]
function DispatchMessage(MsgKey, MsgData)
	if (DEV_MSG[MsgKey] ~= nil and (type(DEV_MSG[MsgKey]) == "function")) then
		LogInfo("DEV_MSG." .. tostring(MsgKey) .. ":  " .. tostring(MsgData))
		local status, err = pcall(DEV_MSG[MsgKey], MsgData)
		if (not status) then
			LogError("LUA_ERROR: " .. err)
		end
	else
		LogTrace("HandleMessage: Unhanded command = " .. MsgKey)
	end
end

--[[
TODO: Create DEV_MSG functions for all messages to call Notifies.
	  Sample functions are included below for all applicable notifications.
--]]

-- POWER
function DEV_MSG.p(value)
	LogTrace("DEV_MSG.POWER(), value = " .. value)
	local output = 1 	--mod 1000 value of Output Connection ID
	
	if (value == "1") then
		gReceiverProxy:dev_PowerOn(output)
	elseif (value== "0") then
		gReceiverProxy:dev_PowerOff(output)
	else
		LogWarn("DEV_MSG.POWER(): value not valid - " .. value)
	end		
end

-- INPUT
function DEV_MSG.i(value)
	LogTrace("DEV_MSG.i(), value = " .. value)
	local output = 1 	--mod 1000 value of Output Connection ID	
	local input = value 	--mod 1000 value of Input Connection ID
	local bindingID = 5001
	
	LogInfo("INPUT_OUTPUT_CHANGED, input = " .. value)
	gReceiverProxy:dev_InputOutputChanged(bindingID, input, output)
end

-- VOLUME
function DEV_MSG.v(value)
	LogTrace("DEV_MSG.v(), value = " .. value)
	
	local output = 1 --mod 1000 value of Output Connection ID
	local deviceLevel = value	
	local c4Level = deviceLevel	
	
	gReceiverProxy:dev_VolumeLevelChanged(output, c4Level, deviceLevel)		
end

-- MUTE
function DEV_MSG.m(value)
	LogTrace("DEV_MSG.m(), value = " .. value)
	
	local output = 1 --mod 1000 value of Output Connection ID
	local state   	--Mute state represented as "True" or "False"
	if (value == "1") then 
		state = "True"
	elseif (value == "0") then
		state = "False"
	else
		LogWarn("DEV_MSG.MUTE(), value not valid, exiting...")
		return
	end
	gReceiverProxy:dev_MuteChanged(output, state)
end	
	
